unit hiPendulum;

interface

uses Kol,Share,Debug;

type
  THIPendulum = class(TDebug)
   private
     FCounter:integer;
     FDirect: byte;
     FunctionDirect: procedure of object;
     ReverseFunctionDirect: procedure of object;     
     procedure SetDirect(Value: byte);
     procedure Increment;
     procedure Decrement;
     procedure Reverse;
   public
    _prop_Min: integer;
    _prop_Max: integer;
    _prop_Step: integer;
        
    _event_onValue: THI_Event;
    _event_onSet: THI_Event;    
    _event_onMaxMin: THI_Event;
    _event_onMinMax: THI_Event;        
    _event_onReverse: THI_Event;
    
    property _prop_Direct: byte read FDirect write SetDirect;

    procedure _work_doNext(var _Data:TData; Index:word);
    procedure _work_doPrev(var _Data:TData; Index:word);
    procedure _work_doReset(var _Data:TData; Index:word);
    procedure _work_doMax(var _Data:TData; Index:word);
    procedure _work_doMin(var _Data:TData; Index:word);
    procedure _work_doValue(var _Data:TData; Index:word);
    procedure _work_doStep(var _Data:TData; Index:word);
    procedure _work_doDirect(var _Data:TData; Index:word);        
    procedure _var_Value(var _Data:TData; Index:word);
    procedure _var_Min(var _Data:TData; Index:word); 
    procedure _var_Max(var _Data:TData; Index:word);     
    procedure _var_Direct(var _Data:TData; Index:word);    
    property _prop_Default:integer write FCounter;
  end;

implementation

procedure THIPendulum.Increment;
begin
  inc(FCounter, _prop_Step);
  if FCounter > _prop_Max then
   begin
    FCounter := _prop_Max;
    dec(FCounter, _prop_Step);
    Reverse;
    _hi_onEvent(_event_onMaxMin); 
   end;
end;

procedure THIPendulum.Decrement;
begin
  dec(FCounter, _prop_Step);
  if FCounter < _prop_Min then
   begin
    FCounter := _prop_Min;
    inc(FCounter, _prop_Step);
    Reverse;
    _hi_onEvent(_event_onMinMax);
   end;
end;

procedure THIPendulum.Reverse;
begin
 if FDirect = 1 then SetDirect(0)
  else SetDirect(1);
 _hi_onEvent(_event_onReverse);
end;

procedure THIPendulum.SetDirect;
begin
  FDirect := Value;
  if FDirect = 0 then
  begin
    FunctionDirect        := Increment;
    ReverseFunctionDirect := Decrement;
  end   
  else
  begin
    FunctionDirect        := Decrement;
    ReverseFunctionDirect := Increment;
  end;            
end;

procedure THIPendulum._work_doNext;
begin
   FunctionDirect;
  _hi_CreateEvent(_Data, @_event_onValue, FCounter);
end;

procedure THIPendulum._work_doPrev;
begin
  ReverseFunctionDirect;
  _hi_CreateEvent(_Data, @_event_onValue, FCounter);
end;

procedure THIPendulum._work_doReset;
begin
  if FDirect = 0 then
    FCounter := _prop_Min
  else
    FCounter := _prop_Max;
  _hi_CreateEvent(_Data, @_event_onSet, FCounter);    
end;

procedure THIPendulum._work_doValue;
begin
  FCounter := ToInteger(_Data);
  _hi_CreateEvent(_Data, @_event_onSet, FCounter);
end;

procedure THIPendulum._work_doMax;
begin
  _prop_Max := ToInteger(_Data);
end;

procedure THIPendulum._work_doMin;
begin
  _prop_Min := ToInteger(_Data);
end;

procedure THIPendulum._work_doStep;
begin
  _prop_Step := ToInteger(_Data);
end;

procedure THIPendulum._work_doDirect;
begin
  _prop_Direct := ToInteger(_Data);
end;

procedure THIPendulum._var_Value;
begin
  dtInteger(_Data, FCounter);
end;

procedure THIPendulum._var_Direct;
begin
  dtInteger(_Data, FDirect);
end;

procedure THIPendulum._var_Max;
begin
  dtInteger(_Data, _prop_Max);
end;

procedure THIPendulum._var_Min;
begin
  dtInteger(_Data, _prop_Min);
end;

end.
